package com.xperia64.diyedit.metadata;

import com.xperia64.diyedit.FileByteOperations;

// Generic metadata class, works for all filetypes
public class Metadata {
	
	public byte[] file;
	public Metadata(String f)
	{
		file = FileByteOperations.read(f);
	}
	public Metadata(byte[] b)
	{
		file=b;
	}
	// Returns the name in the mio file header
	public String getName()
	{
		String name="";
		for(int i = 28; i<40; i++)
		{
			if(file[i]==0)
			{
				break;
			}
			name+=(char)file[i];
		}
		return name;
	}
	// Sets the name in the mio file header
	public void setName(String name)
	{
		char[] c = name.toCharArray();
		for(int i = 28; i<40; i++)
		{
			if(i-28<c.length)
			{
				file[i]=(byte)c[i-28];
			}else{
				file[i]=0x00;
			}
		}
	}
	// Returns the description from the mio file header
	public String getDescription()
	{
		String desc="";
		boolean finished=false;
		for(int i = 91; i<127; i++)
		{
			if(file[i]==0)
			{
				finished=true;
			}
			if(!finished)
			{
			desc+=(char)file[i];
			}
		}
		return desc;
	}
	// Sets the description in the mio file header
	public void setDescription(String desc)
	{
		char[] d = desc.toCharArray();
		for(int i = 91; i<127; i++)
		{
			if(i-91<d.length)
			{
				file[i]=(byte)d[i-91];
			}else{
				file[i]=0x00;
			}
		}
	}
	// Returns the brand from the mio file header
	public String getBrand()
	{
		String brand="";
		boolean finished=false;
		for(int i = 53; i<62; i++)
		{
			if(file[i]==0)
			{
				finished=true;
			}
			if(!finished)
			{
				brand+=(char)file[i];
			}
			
		}
		return brand;
	}
	// Sets the brand in the mio file header
	public void setBrand(String brand)
	{
		char[] b = brand.toCharArray();
		for(int i = 53; i<62; i++)
		{
			if(i-53<b.length)
			{
				file[i]=(byte)b[i-53];
			}else{
				file[i]=0x00;
			}
		}
	}
	// Returns the creator from the mio file header
	public String getCreator()
	{
		String cr="";
		boolean finished=false;
		for(int i = 72; i<81; i++)
		{
			if(file[i]==0)
			{
				finished=true;
			}
			if(!finished)
			{
				cr+=(char)file[i];
			}
			
		}
		return cr;
	}
	// Sets the creator in the mio file header
	public void setCreator(String cre)
	{
		char[] cr = cre.toCharArray();
		for(int i = 72; i<81; i++)
		{
			if(i-72<cr.length)
			{
				file[i]=(byte)cr[i-72];
			}else{
				file[i]=0x00;
			}
		}
	}
	// Returns the first 4-character serial from the mio file header
	public String getSerial1()
	{
		String s1="";
		for(int i = 207; i<211; i++)
		{
			s1+=(char)file[i];
		}
		return s1;
	}
	// Returns the second 4-digit serial from the mio file header
	public int getSerial2()
	{
		int s2=0;
		String tmp=Integer.toHexString(byteForAddition(file[215]));
		while(tmp.length()<2)
		{
			tmp="0"+tmp;
		}
		if(tmp.length()>2)
		{
			tmp=tmp.substring(tmp.length()-2,tmp.length());
		}
		String tmp2=Integer.toHexString(byteForAddition(file[214]));
		while(tmp2.length()<2)
		{
			tmp2="0"+tmp2;
		}
		String tmp3=Integer.toHexString(byteForAddition(file[213]));
		if(tmp2.length()>2)
		{
			tmp2=tmp2.substring(tmp2.length()-2,tmp2.length());
		}
		while(tmp3.length()<2)
		{
			tmp3="0"+tmp3;
		}
		if(tmp3.length()>2)
		{
			tmp3=tmp3.substring(tmp3.length()-2,tmp3.length());
		}
		String tmp4=Integer.toHexString(byteForAddition(file[212]));
		while(tmp3.length()<2)
		{
			tmp3="0"+tmp3;
		}
		s2=Integer.parseInt((tmp+tmp2+tmp3+tmp4),16)+1;
		return s2;
	}
	// Returns the third 3-digit serial from the mio file header
	public int getSerial3()
	{
		int s3=0;
		String tmp=Integer.toHexString(byteForAddition(file[217]));
		tmp+=Integer.toHexString(byteForAddition(file[216]));
		s3=Integer.parseInt(tmp,16);
		return s3;
	}
	// Sets the entire mio file serial
	public void setSerial(String s1, int s2, int s3)
	{
		char[] c = s1.toCharArray();
		s2=s2-1;
		for(int i = 207; i<211; i++)
		{
			file[i]=(byte) c[i-207];
		}
		String hex11=Integer.toHexString(s2);
		int l = hex11.length();
		for(int i = 0; i<8-l; i++)
		{
			hex11="0"+hex11;
		}
		String hex21=Integer.toHexString(s3);
		l=hex21.length();
		for(int i = 0; i<4-l; i++)
		{
			hex21="0"+hex21;
		}
		file[212]=(byte) Integer.parseInt(hex11.substring(6,8),16);
		file[213]=(byte) Integer.parseInt(hex11.substring(4,6),16);
		file[214]=(byte) Integer.parseInt(hex11.substring(2,4),16);
		file[215]=(byte) Integer.parseInt(hex11.substring(0,2),16);
		file[216]=(byte) Integer.parseInt(hex21.substring(2,4),16);
		file[217]=(byte) Integer.parseInt(hex21.substring(0,2),16);
	}
	// Returns if the game is locked or not
	public boolean getLocked()
	{
		if(file[232]!=0)
		{
			return true;
		}
		return false;
	}
	// Sets whether the game is locked or not
	public void setLocked(boolean lock)
	{
		if(lock)
		{
			file[232]=5; // Arbitrary non-zero value unlocks.
		}else{
			file[232]=0;
		}
	}
	// Fix bytes because -128 is needed for some reason in java
		private int byteForAddition(byte b)
		{
			int[] negBytes=new int[128];
			int pos=0;
			for(int i = 128; i>=1; i--)
			{
				negBytes[pos]=i;
				pos++;
			}
			int b2=b;
			if(b<0)
			{
				b2=127+negBytes[(b*-1)-1];
			}
			return b2;
			
		}
		public byte[] returnChanges()
		{
			return file;
		}
}
